#ifndef LOAN_H
#define LOAN_H

#include <FL/Fl_Check_Button.H>
#include "Borrower.h"
#include "Lender.h"
#include "Bundle.h"
#include "Selection.h"
#include <stdio.h>
#include "SQL_UTIL.h"
#include <fstream>

class Loan : public Fl_Check_Button {
 
 public:
  int loanID;
  int candidate;
  bool bundled;
  float originalValue;
  float principalRemaining;
  float profit;
  float discountRate;
  float interestRate;
  bool isARM;
  char* settlementDate;
  int term;
  int risk;
  int status;
  int lenderID;
  bool isJumbo;
  double propertyID;
  char* buttonlabel;

  static Loan* displayLoans( Selection* , int);
  //static Loan* displayCandLoans();

  Loan *prev;
  Loan *next;

  Loan( int loan_id, int lender_id, float original_amount, float prin_remain,
        float int_rate, bool rate_type,char* sett_date, bool loan_type,
        int loan_term, int loan_status, int cand, bool bun, int loan_risk, 
	float loan_profit,float disc_rate, Loan* previous ) 
    : Fl_Check_Button( 15, 75, 400, 75, " " )     {
    
    loanID = loan_id;
    lenderID = lender_id;
    originalValue=original_amount;
    principalRemaining = prin_remain;
    interestRate = int_rate;
    isARM = rate_type;
    status = loan_status;

    settlementDate = new char[40];
    strcpy(settlementDate,sett_date);

    isJumbo = loan_type;
    term = loan_term;
    candidate = cand;
    bundled = bun;
    risk = loan_risk;
    profit = loan_profit;
    discountRate = disc_rate;

    prev = previous;
    next = NULL;

    char irtype[4];

    if( rate_type ) {
      strcpy(irtype,"FM");
    } else {
      strcpy(irtype,"ARM");
    }

    char loansize[8];

    if( isJumbo ) {
      strcpy( loansize, "Regular" ); // logic reversed
    } else {
      strcpy( loansize, "Jumbo" );
    }
    
    char* l_status;
    if(status==0) l_status = "Good";
    if(status==1) l_status = "Late";
    if(status==2) l_status = "Default";

    char *buttonlabel = new char[500];
    if( candidate != -1 ) {
      sprintf( buttonlabel, "UserID:%d   ID:%d   LenderID:%d  Value:$%.2f\n"
                            "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                            "Settlement Date:%s   Term:%d   Status=%s\n"
                            "DiscountRate:$%.2f   PrincipalRem:$%.2f",
               candidate, loanID, lenderID, originalValue, loansize, risk,
               interestRate, irtype, settlementDate, term, l_status,
               discountRate, principalRemaining                              );
      //printf("\n%s\n",buttonlabel);
      label(buttonlabel);

      if( bundled ) {
	labelcolor(FL_DARK_GREEN);
      } else {
	labelcolor(FL_DARK_RED);
      }
    } else {

      sprintf( buttonlabel, "ID:%d   LenderID:%d   Value:$%.2f\n"
                            "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                            "Settlement Date:%s   Term:%d\n"
                            "DiscountRate:$%.2f   PrincipalRem:$%.2f",
               loanID, lenderID, originalValue, loansize, risk,
               interestRate, irtype, settlementDate, term, discountRate,
               principalRemaining                                        );

      label(buttonlabel);
    }

  };

  Loan() : Fl_Check_Button(15,65,400,50,"") { 

    next = NULL;
    prev = NULL;

  };

  void writeLoanReport(Loan* list){

    ofstream fout("loans_report.txt");
    char output[500];

    while(list!=NULL){

      char irtype[4];

      if( !list->isARM ) {
        strcpy(irtype,"FM");
      } else {
        strcpy(irtype,"ARM");
      }

      char loansize[8];

      if( list->isJumbo ) {
        strcpy( loansize, "Regular" ); // logic reversed
      } else {
        strcpy( loansize, "Jumbo" );
      }
      sprintf( output, "ID:%d   LenderID:%d   Value:$%.2f\n"
                       "Size:%s   Risk:%d   IR:%.3f   IRType:%s\n"
                       "Settlement Date:%s   Term:%d\n"
                       "DiscountRate:$%.2f   PrincipalRem:$%.2f\n",
               list->loanID, list->lenderID, list->originalValue,
	       loansize, list->risk, list->interestRate, irtype,
               list->settlementDate, list->term, list->discountRate,
               list->principalRemaining                              );
      
      fout << output << endl;
      list = list->next;
    }
    fout.close();
  }

};

Loan* get_selected_loans( const char*,int);

int create_bundle(char*,float,int,float);

void bundle_loans(Loan*,int);

Loan* optimize(Loan*,int,int,float,float);

Bundle* getBundles();

Loan* displayCandLoans();

bool mark_loan(int);

void unmark_loan(int);
#endif


